Attribute VB_Name = "Rules"
Const MODULENAME = "Rules"
'defining cards` values constants
Const KING = 0
Const ACE = 1
Const JACK = 11
Const QUEEN = 12
Const TEN = 10

'defining the possible sequences
Global Const ROYAL_FLUSH = 10
Global Const STRAIGHT_FLUSH = 9
Global Const FOUR_OF_AKIND = 8
Global Const FULL_HOUSE = 7
Global Const FLUSH = 6
Global Const STRAIGHT = 5
Global Const THREE_OF_AKIND = 4
Global Const TWO_PAIR = 3
Global Const JACKS_OR_BETTER = 2
Global Const TENS_OR_BETTER = 1
Global Const NONE = 0


Global Const PLAYER = 0
Global Const HOUSE = 1
Global Const TIE = -1

'this function determines who is the session winner according to the
'given consequences
'mConseq1 - the user`s consequence
'mConseq2 - the computer`s consequence
'returned value :  0 - the player won
'                  1 - the HOUSE won
'                 -1 - tie
Public Function SessionWinner(ByVal mConseq1 As Integer, ByVal mConseq2 As Integer) As Integer
If mConseq1 > mConseq2 Then
    SessionWinner = PLAYER
    Exit Function
ElseIf mConseq1 < mConseq2 Then
    SessionWinner = HOUSE
    Exit Function
Else
    SessionWinner = TIE
End If
End Function

'this sub determines the number of times a card kind is repeated
'in  a group of cards.
'mnKind - an array which the sub uses to store the number of times
'each kind is repeated. for instance mnKind(1)=2 means there are
'two aces in the group of cards given.
'Hnd - an array of pointers to the Qcards which the group consists of.
'Qcrds - Qcards object
Private Sub nOfKind(ByRef mnKind() As Integer, Hnd() As String, Qcrds As Qcards)
Dim IntI, IntJ, nCrd1, nCrd2 As Integer

For IntI = 1 To UBound(Hnd)
'this expression maps the card number to the range of 0-12 according
'to the following table:
'nCrd1 | card`s kind
'---------------------
'   0  | King
'   1  | Ace
'   2  |  2
'   3  |  3
'  4-10|  4-10
'  11  | Jack
'  12  | Queen

    nCrd1 = Qcrds.Item(Hnd(IntI)).nCard Mod 13
    If mnKind(nCrd1) = 0 Then ' if a new kind of card is found
        For IntJ = IntI To UBound(Hnd) ' find how much times he is repeated
            nCrd2 = Qcrds.Item(Hnd(IntJ)).nCard Mod 13
            mnKind(nCrd1) = mnKind(nCrd1) + Abs(nCrd1 = nCrd2)
        Next
    End If
Next
End Sub
'this sub determines if a group of cards represents a flush sequence-
'all the cards are having the same sign.
'Hnd - an array of pointers to the Qcards which the group consists of.
'Qcrds - Qcards object
'returned value : true - flush sequence was found
'                 false - flush sequence was not found
Private Function IsFlush(Hnd() As String, Qcrds As Qcards) As Boolean
Dim IntI, nCrd, nSign As Integer

'finding the first card`s sign
nSign = Qcrds.Item(Hnd(1)).Sign

'matching the first card`s sign to the other cards` sign
'if one sign does not match then return false else return true
For IntI = 2 To 5
    nCrd = Qcrds.Item(Hnd(IntI)).Sign
    If nCrd <> nSign Then
        IsFlush = False
        Exit Function
    End If
Next

IsFlush = True
End Function

'this function checks if a group of cards represents
' a 4 cards flush sequence, and if it does it marks the cards
' which consists the 4 cards flush sequence. the function is used
' to determine the computer strategy in the game .
'Hnd - an array of pointers to the Qcards which the group consists of.
'Qcrds - Qcards object
'bHold - an array used to mark the cards which consists the 4 cards flush
'sequence
'returned value: true - a 4 cards flush sequence was found
'                false - a 4 card flush sequence was not found
Private Function IsFourCardsOfFlush(Hnd() As String, Qcrds As Qcards, bHold() As Boolean) As Boolean
Dim IntI As Integer

'Sign(4) - an array which the function uses to store the number of times
'each suit is repeated in the group. for instance Sign(3)=2 means there are
'two hearts in the group of cards given.
Dim Sign(4) As Integer

'the bHold array size must be equal or bigger to the Hnd array
If UBound(bHold) < UBound(Hnd) Then
    MsgBox "Invalid use of IsFourCardsOfFlush " _
         , vbExclamation + vbOKOnly, MODULENAME
    Exit Function
End If

'initialization of the array , assuming a flush sequence is found
For IntI = 1 To UBound(bHold)
    bHold(IntI) = True
Next

'finding the number of occurrences  of each suit in the group
For IntI = 1 To UBound(Hnd)
    Sign(Qcrds.Item(Hnd(IntI)).Sign) = Sign(Qcrds.Item(Hnd(IntI)).Sign) + 1
Next

For IntI = 1 To 4
    If Sign(IntI) = 4 Then ' is a 4 card flush sequence was found?
        For IntJ = 1 To 5 ' finding the card which suit don`t match the other cards.
            If Qcrds.Item(Hnd(IntJ)).Sign <> IntI Then
                bHold(IntJ) = False
                IsFourCardsOfFlush = True
                Exit Function
            End If
        Next
    End If
Next

'a 4 card flush sequence was not found
For IntI = 1 To 5
    bHold(IntI) = False
Next
Is4CardsOfFlush = False
End Function

'this function determines if a group of cards represents a straight
'sequence.
'Hnd - an array of pointers to the Qcards which the group consists of.
'Qcrds - Qcards object
'returned value: true - a straight sequence was found
'                false - a straight sequence was not found
Private Function IsStraight(Hnd() As String, Qcrds As Qcards) As Boolean
Dim IntI, mnKind(13) As Integer
Dim nCrdsPos(13) As Boolean

'verifying that each card value in the group is unique using 'nOfKind'
'in case the cards` value is not unique - a straight is not found
nOfKind mnKind(), Hnd(), Qcrds
For IntI = 0 To 12
    If mnKind(IntI) > 1 Then
        IsStraight = False
        Exit Function
    End If
Next

'mapping the cards` values in an array which describes the position
'of each card as in the following example:
' the cards in the group are : King , Queen , 6, 7, 8
' the nCrdPos array will be populated thus -
'index: 0 1 2 3 4 5 6 7 8 9 10 11 12
'value: 1 0 0 0 0 0 1 1 1 0  0  0  1
For IntI = 1 To 5
    nCrdsPos(Qcrds.Item(Hnd(IntI)).nCard Mod 13) = True
Next

'if a King value is not present in the group
If nCrdsPos(0) <> True Then
'try to find the first card in a possible straight
    For IntI = 1 To 8
        If nCrdsPos(IntI) = True Then
'find if this card is the first of 5 in a straight
            For IntJ = IntI To IntI + 4
                If nCrdsPos(IntJ) <> True Then
                    IsStraight = False
                    Exit Function
                End If
            Next

           Exit For
        End If
    Next
Else
'King value is present in the group, find the cards above the
'king (ace, 2 , 3) which consists a possible straight
    For IntI = 1 To 4
        If nCrdsPos(IntI) <> True Then
'find the cards below the king (10, jack , queen) which will complete
'a possible straight
'for example: if 2 cards were found in a row above the king (ace and 2)
'then 2 cards below the king should be found (queen and jack)  in
'order to complete a straight
            For IntJ = 12 To IntI + 8 Step -1
                If nCrdsPos(IntJ) <> True Then
                    IsStraight = False
                    Exit Function
                End If
            Next
        Exit For
        End If
    Next
End If

'a straight was found!


'Debug.Print nCrds(0) & " " & nCrds(1) & " " & nCrds(2) & " " & nCrds(3) & " " & nCrds(4) & " " & nCrds(5)

IsStraight = True
End Function

'this function searches through a group of Qcards for a
'valid sequence of cards according to the rules of poker
'game.
'Hnd - an array of pointers to the Qcards which the group consists of.
'Qcrds - Qcards object
'retuned value- an integer which indicates the sequence that
'was found according to the constants defined above
Public Function CardsConsequence(Hnd() As String, Qcrds As Qcards) As Integer
Dim bStraight, bFlush, bRoyal As Boolean
Dim Qcrd As New Qcard
Dim IntI, IntJ, PairValue, mnKind(13) As Integer

'searching through the Qcards to find a straight sequence
bStraight = IsStraight(Hnd(), Qcrds)
'searching through the Qcards to find a flush sequence
bFlush = IsFlush(Hnd(), Qcrds)

'searching for a royal flush consequence
If bStraight And bFlush Then
'assuming a royal flush was found
    bRoyal = True
    For IntI = 1 To 5
'assuring that the cards that build the sequence are: Ten,Jack,Queen,King,Ace
'if one of the cards is not from the above then bRoyal is set to FALSE
        bRoyal = bRoyal And (Qcrds.Item(Hnd(IntI)).nCard Mod 13 >= 10 Or Qcrds.Item(Hnd(IntI)).nCard Mod 13 <= 1)
    Next
'if bRoyal is TRUE then a Royal Flush was found!
    If bRoyal Then
         CardsConsequence = ROYAL_FLUSH
    Else
'bRoyal is FALSE, a Straight Flush was found!
         CardsConsequence = STRAIGHT_FLUSH
    End If
    Exit Function
End If

'the following code will be executed in case Royal Flush
'and Straight Flush were not found.

'finding the number of times a card kind is repeated
'in  the group of cards.
nOfKind mnKind, Hnd, Qcrds

'Iterating through the mnKind array
For IntI = 0 To 12
    If mnKind(IntI) = 4 Then
' a four of a kind sequence was found!
            CardsConsequence = FOUR_OF_AKIND
        Exit Function
    End If
'searching for a full house sequence
    If mnKind(IntI) = 3 Or mnKind(IntI) = 2 Then
        For IntJ = IntI + 1 To 12
            If mnKind(IntJ) = 5 - mnKind(IntI) Then
                CardsConsequence = FULL_HOUSE
                Exit Function
            End If
        Next
    End If
Next

'the following code will be executed in case Royal Flush
', Straight Flush ,four of a kind, full house were not found.

'searching for a flush sequence
If bFlush Then
'a flush sequence was found!
    CardsConsequence = FLUSH
    Exit Function
End If
'searching for a straight sequence
If bStraight Then
'a straight sequence was found!
    CardsConsequence = STRAIGHT
    Exit Function
End If

'the following code will be executed in case Royal Flush
', Straight Flush ,four of a kind, full house,flush,straight
'were not found.

'Iterating through the mnKind array
For IntI = 0 To 12
'searching for a three of a kind sequence
    If mnKind(IntI) = 3 Then
'a three of a kind sequence was found!
        CardsConsequence = THREE_OF_AKIND
        Exit Function
    End If
'searching for a two pair sequence
    If mnKind(IntI) = 2 Then
'a pair was found, now search for another one to complete a two pair
'sequence
        For IntJ = IntI + 1 To 12
            If mnKind(IntJ) = 2 Then
'a two pair sequence was found!
                CardsConsequence = TWO_PAIR
                Exit Function
            End If
        Next
'searching for a jack or better sequence
        PairValue = IntI
        If PairValue = KING Or PairValue = JACK Or PairValue = QUEEN _
            Or PairValue = ACE Then
'a jack or better sequence was found!
                CardsConsequence = JACKS_OR_BETTER
                Exit Function
        End If
    End If
Next
'the following code will be executed in case Royal Flush
', Straight Flush ,four of a kind, full house,flush,straight
',three of a kind,two pair,jacks or better were not found.

'none of the valid sequences in poker game was found :-(
CardsConsequence = NONE
End Function
'this function searches through a group of Qcards and
'chooses the valuable cards to hold according to an
'implemented strategy.
'Hnd - an array of pointers to the Qcards which the group consists of.
'Qcrds - Qcards object
'bHold - an array used to mark the valuable cards
Public Sub ChooseHoldCards(Hnd() As String, Qcrds As Qcards, bHold() As Boolean)
Dim bStraight, bFlush, bRoyal As Boolean
Dim Qcrd As New Qcard
Dim IntI, IntJ, IntK, PairValue, nSign, mnKind(13) As Integer

PairValue = -1
For IntI = 1 To 5
    bHold(IntI) = False
Next

'searching through the Qcards to find a straight sequence
bStraight = IsStraight(Hnd(), Qcrds)
'searching through the Qcards to find a flush sequence
bFlush = IsFlush(Hnd(), Qcrds)

'searching for a royal flush consequence
If bStraight And bFlush Then
'assuming a royal flush was found
    bRoyal = True
    For IntI = 1 To 5
'assuring that the cards that build the sequence are: Ten,Jack,Queen,King,Ace
'if one of the cards is not from the above then bRoyal is set to FALSE
        bRoyal = bRoyal * (Qcrds.Item(Hnd(IntI)).nCard Mod 13 >= 10 Or Qcrds.Item(Hnd(IntI)).nCard Mod 13 <= 1)
    Next
'if bRoyal is TRUE then a Royal Flush was found!
    If bRoyal Then
'a Royal flush was found -> Hold all the cards
        For IntI = 1 To 5
            bHold(IntI) = True
        Next
    Else
'bRoyal is FALSE, a Straight Flush was found!-> Hold all the cards
        For IntI = 1 To 5
            bHold(IntI) = True
        Next
    End If
    Exit Sub
End If

'the following code will be executed in case Royal Flush
'and Straight Flush were not found.

'finding the number of times a card kind is repeated
'in  the group of cards.
nOfKind mnKind, Hnd, Qcrds
For IntI = 0 To 12
    If mnKind(IntI) = 4 Then
' a four of a kind sequence was found!-> hold all the cards
        For IntJ = 1 To 5
            bHold(IntI) = True
        Next
        Exit Sub
    End If
'searching for a full house sequence
    If mnKind(IntI) = 3 Or mnKind(IntI) = 2 Then
        For IntJ = IntI + 1 To 12
            If mnKind(IntJ) = 5 - mnKind(IntI) Then
' a full house sequence was found!-> hold all the cards
                For IntK = 1 To 5
                    bHold(IntI) = True
                Next
                Exit Sub
            End If
        Next
    End If
Next

'the following code will be executed in case Royal Flush
', Straight Flush ,four of a kind, full house were not found.

'searching for a flush sequence
If bFlush Then
'a flush sequence was found!-> hold all the cards
    For IntI = 1 To 5
        bHold(IntI) = True
    Next
    Exit Sub
End If

'searching for a straight sequence
If bStraight Then
'a straight sequence was found!-> hold all the cards
    For IntI = 1 To 5
        bHold(IntI) = True
    Next
    Exit Sub
End If

'the following code will be executed in case Royal Flush
', Straight Flush ,four of a kind, full house,flush,straight
'were not found.

'the following lines implement the rule:
'If you have 4 cards of a straight flush, draw 1 card, but do not
'break up a straight or a flush to do so.
If IsFourCardsOfStraight(Hnd(), Qcrds, bHold()) Then
'if a straight was found then check if the straight consists
'of the same suit

'one of two cards in the hand belongs to the straight
    For IntI = 1 To 2
        If bHold(IntI) Then
'if the card belong to the straight then hold its suit in nSign
            nSign = Qcrds.Item(Hnd(IntI)).Sign
        End If
    Next
'find how much cards in the straight have the nSign suit
    For IntI = 1 To 5
        If bHold(IntI) Then
            nSameSign = nSameSign + Abs(nSign = Qcrds.Item(Hnd(IntI)).Sign)
        End If
    Next
    If nSameSign = 4 Then
'4 cards of straight flush!
'the 'IsFourCardsOfStraight' function marks the relevant cards
        Exit Sub
    Else
'unmark the cards marked by the 'IsFourCardsOfStraight' function
        For IntI = 1 To 5
             bHold(IntI) = False
        Next
    End If
End If

'Iterating through the mnKind array
For IntI = 0 To 12
'searching for a three of a kind sequence
    If mnKind(IntI) = 3 Then
'a three of a kind sequence was found!->hold these cards
     
        For IntJ = 1 To 5
            If (Qcrds.Item(Hnd(IntJ)).nCard Mod 13) = IntI _
                Then bHold(IntJ) = True
        Next
        Exit Sub
    End If
'searching for a two pair sequence
    If mnKind(IntI) = 2 Then
'a pair was found, now search for another one to complete a two pair
'sequence
        For IntJ = IntI + 1 To 12
            If mnKind(IntJ) = 2 Then
'a two pair sequence was found!->hold the two pair
                For IntK = 1 To 5
                    If (Qcrds.Item(Hnd(IntK)).nCard Mod 13) = IntI _
                        Or (Qcrds.Item(Hnd(IntK)).nCard Mod 13) = IntJ _
                             Then bHold(IntK) = True
                Next
                Exit Sub
            End If
        Next
        PairValue = IntI
        If PairValue = KING Or PairValue = JACK Or PairValue = QUEEN _
            Or PairValue = ACE Then
'a jack or better sequence was found!->hold the jack
                For IntJ = 1 To 5
                    If (Qcrds.Item(Hnd(IntJ)).nCard Mod 13) = IntI _
                        Then bHold(IntJ) = True
                Next
                Exit Sub
        End If
    End If
Next
'the following code will be executed in case Royal Flush
', Straight Flush ,four of a kind, full house,flush,straight
',three of a kind,two pair,jacks or better were not found.

'the following lines implement the rule:
'If you have 4 cards of a flush, draw 1 card.
If IsFourCardsOfFlush(Hnd(), Qcrds, bHold()) Then Exit Sub

'the following lines implement the rule:
'If you have any pair, draw 3 cards.
If PairValue <> -1 Then
     For IntJ = 1 To 5
        If (Qcrds.Item(Hnd(IntJ)).nCard Mod 13) = PairValue _
            Then bHold(IntJ) = True
     Next
     Exit Sub
End If

'the following lines implement the rule:
'If you have 4 cards of a straight, draw 1 card.
If IsFourCardsOfStraight(Hnd(), Qcrds, bHold()) Then Exit Sub

End Sub
Private Function IsFourCardsOfStraight(Hnd() As String, Qcrds As Qcards, bHold() As Boolean) As Boolean
Const ORG_POS = 0
Const CARD_VAL = 1
Const DIFFERENCE = 2
Dim IntI, nPass, nTemp, nMaxCard, nMult As Integer
Dim bNoExchanges As Boolean
Dim mHnd() As Integer
ReDim mHnd(2, UBound(Hnd))
For IntI = 0 To UBound(Hnd) - 1
    mHnd(ORG_POS, IntI) = IntI + 1
    mHnd(CARD_VAL, IntI) = Qcrds(Hnd(IntI + 1)).nCard Mod 13
Next

'bubble sort
nPass = 1
Do
    bNoExchanges = True
    For IntI = 0 To UBound(Hnd) - 1 - nPass
        If mHnd(CARD_VAL, IntI) < mHnd(CARD_VAL, IntI + 1) Then
            nTemp = mHnd(CARD_VAL, IntI)
            mHnd(CARD_VAL, IntI) = mHnd(CARD_VAL, IntI + 1)
            mHnd(CARD_VAL, IntI + 1) = nTemp
            nTemp = mHnd(ORG_POS, IntI)
            mHnd(ORG_POS, IntI) = mHnd(ORG_POS, IntI + 1)
            mHnd(ORG_POS, IntI + 1) = nTemp
            bNoExchanges = False
        End If
       
    Next
    nPass = nPass + 1
Loop While Not bNoExchanges

If mHnd(CARD_VAL, 3) < 2 Then
    nMaxCard = 3
ElseIf mHnd(CARD_VAL, 4) < 2 Then
    nMaxCard = 4
Else
    nMaxCard = 0
End If

Debug.Print mHnd(CARD_VAL, 0) & " " & mHnd(CARD_VAL, 1) & " " & mHnd(CARD_VAL, 2) & " " & mHnd(CARD_VAL, 3) & " " & mHnd(CARD_VAL, 4)

For IntI = 0 To UBound(Hnd) - 1
    mHnd(DIFFERENCE, IntI) = mHnd(CARD_VAL, IntI) - mHnd(CARD_VAL, (IntI + 1) Mod 5)
    If mHnd(DIFFERENCE, IntI) = -12 Then mHnd(DIFFERENCE, IntI) = 1
    If mHnd(DIFFERENCE, IntI) = -11 Then mHnd(DIFFERENCE, IntI) = 2
Next
Debug.Print mHnd(DIFFERENCE, 0) & " " & mHnd(DIFFERENCE, 1) & " " & mHnd(DIFFERENCE, 2) & " " & mHnd(DIFFERENCE, 3) & " " & mHnd(DIFFERENCE, 4)

For IntI = nMaxCard To nMaxCard + 4
    If mHnd(DIFFERENCE, IntI Mod 5) = 2 Or mHnd(DIFFERENCE, IntI Mod 5) = 1 Then
        nMult = mHnd(DIFFERENCE, IntI Mod 5) * mHnd(DIFFERENCE, (IntI + 1) Mod 5) * mHnd(DIFFERENCE, (IntI + 2) Mod 5)
        If nMult = 2 Or nMult = 1 Then
          '  MsgBox "4cardsofstraight"
            bHold(mHnd(ORG_POS, IntI Mod 5)) = True
            bHold(mHnd(ORG_POS, (IntI + 1) Mod 5)) = True
            bHold(mHnd(ORG_POS, (IntI + 2) Mod 5)) = True
            bHold(mHnd(ORG_POS, (IntI + 3) Mod 5)) = True
           
            IsFourCardsOfStraight = True
            Exit Function
        End If
    End If
Next
IsFourCardsOfStraight = False

End Function

